@@ -5,10 +5,10 @@ use strict;
 use warnings;
 
 package Log::Log4perl::Appender::RabbitMQ;
-$Log::Log4perl::Appender::RabbitMQ::VERSION = '0.141470';
+$Log::Log4perl::Appender::RabbitMQ::VERSION = '0.200001';
 our @ISA = qw/ Log::Log4perl::Appender /;
 
-use Net::AMQP::RabbitMQ 0.004002;
+use Net::AMQP::RabbitMQ 2.30000;
 use Readonly;
 
 Readonly my $CHANNEL => 1;
@@ -26,16 +26,6 @@ sub new {
         require Test::Net::RabbitMQ;
     }
 
-    my $self = bless {
-        host        => $args{host}        || 'localhost',
-        routing_key => $args{routing_key} || '%c'       ,
-        declare_exchange => $args{declare_exchange},
-    }, $class;
-
-    # set a flag that tells us to do routing_key interpolation
-    # only if there are things to interpolate.
-    $self->{interpolate_routing_key} = 1 if $self->{routing_key} =~ /%c|%p/;
-
     # Store any given exchange options for declaring an exchange
     my %exchange_options;
     for my $name (qw/
@@ -49,7 +39,6 @@ sub new {
         (my $declare_param_name = $name) =~ s/(.*)_exchange$/$1/;
         $exchange_options{$declare_param_name} = $args{$name} if exists $args{$name};
     }
-    $self->{exchange_options} = \%exchange_options;
 
     # Store any given publish options for use when log is called
     my %publish_options;
@@ -60,7 +49,6 @@ sub new {
     /) {
         $publish_options{$name} = $args{$name} if exists $args{$name};
     }
-    $self->{publish_options} = \%publish_options;
 
     # use any given connect options in connect
     my %connect_options;
@@ -72,18 +60,41 @@ sub new {
        channel_max
        frame_max
        heartbeat
+       ssl
+       ssl_verify_host
+       ssl_cacert
+       ssl_init
     /) {
         $connect_options{$name} = $args{$name} if exists $args{$name};
     }
 
+    # this can be created once as there are no parameters
+    my $mq = $RabbitMQClass->new();
+
+    my $self = bless {
+        host        => $args{host}        || 'localhost',
+        routing_key => $args{routing_key} || '%c'       ,
+        declare_exchange => $args{declare_exchange},
+        connect_options  => \%connect_options,
+        exchange_options => \%exchange_options,
+        publish_options  => \%publish_options,
+        mq               => $mq,
+        _is_connected    => 0,
+    }, $class;
+
+    # set a flag that tells us to do routing_key interpolation
+    # only if there are things to interpolate.
+    $self->{interpolate_routing_key} = 1 if $self->{routing_key} =~ /%c|%p/;
+
     # Create a new connection
     eval {
-        $self->{mq} = _connect_cached($self->{host}, \%connect_options);
+        # connect on construction to make finding errors early easier
+        $self->_connect_cached();
 
         # declare the exchange if declare_exchange is set
-        $self->{mq}->exchange_declare(
-            $CHANNEL, 
-            $self->{publish_options}{exchange}, 
+        $mq->exchange_declare(
+            $CHANNEL,
+            $self->{publish_options}{exchange},
             $self->{exchange_options},
         ) if $self->{declare_exchange};
 
@@ -95,39 +106,23 @@ sub new {
     return $self;
 }
 
-
 ##################################################
-# this closure provides a private class method
-# that will connect to RabbitMQ and cache that
-# connection. The next time it's called with
-# the same params it returns the cached connection.
-{
-    my %connection_cache;
+sub _connect_cached {
+##################################################
+    my $self = shift;
 
-    ##################################################
-    sub _connect_cached {
-    ##################################################
-        my $host = shift;
-        my $connect_options = shift;
+    my $mq = $self->{mq};
 
-        # Create a cache key from the connection options and the host
-        no warnings 'uninitialized';
-        my $cache_key = join(':', $host, sort %$connect_options);
-        use warnings;
-
-        return $connection_cache{$cache_key} if $connection_cache{$cache_key};
-
-        # Create new RabbitMQ object & connection, open channel 1
-        my $mq = $RabbitMQClass->new();
-        $mq->connect($host, $connect_options);
+    if (!$self->{_is_connected} || $$ != $self->{pid}) {
+        #warn "INFO connecting to RabbitMQ\n";
+        $mq->connect($self->{host}, $self->{connect_options});
         $mq->channel_open($CHANNEL);
-
-        # Cache RabbitMQ object
-        $connection_cache{$cache_key} = $mq;
-
-        # Return the RabbitMQ object and the channel we used
-        return $mq;
+        $self->{_is_connected} = 1;
+        # remember pid on connect because forking requires to reconnect
+        $self->{pid} = $$;
     }
+
+    return $mq;
 }
 
 ##################################################
@@ -135,12 +130,7 @@ sub log {
 ##################################################
     my ($self, %args) = @_;
 
-    my $mq = $self->{mq};
-
-    # do nothing if the Net::AMQP::RabbitMQ object is missing
-    return unless $mq;
-
-    # customize the routing key for this message by 
+    # customize the routing key for this message by
     # inserting category and level if interpolate_routing_key
     # flag is set
     my $routing_key = $self->{routing_key};
@@ -149,20 +139,37 @@ sub log {
         $routing_key =~ s/%p/$args{log4p_level}/g;
     }
 
-    # publish the message to the specified group
-    eval {
-        $mq->publish($CHANNEL, $routing_key, $args{message}, $self->{publish_options});
-        1;
-    } or do {
-        # If you got an error warn about it and clear the 
-        # Net::AMQP::RabbitMQ object so we don't keep trying
-        warn "ERROR logging to RabbitMQ via ".ref($self).": $@\n";
-        $self->{mq} = undef;
-    };
+    my $successful = 0;
+    my $try = 0;
+    my $retries = 1;
+    while (!$successful && $try <= $retries) {
+        $try++;
+
+        # publish the message to the specified group
+        eval {
+            my $mq = $self->_connect_cached();
+
+            $mq->publish($CHANNEL, $routing_key, $args{message}, $self->{publish_options});
+            $successful = 1;
+            1;
+        } or do {
+            # If you got an error warn about it and clear the
+            # Net::AMQP::RabbitMQ object so we don't keep trying
+            warn "ERROR logging to RabbitMQ via ".ref($self).": $@\n";
+            # force a reconnect
+            $self->{_is_connected} = 0;
+        };
+    }
 
     return;
 }
 
+sub DESTROY {
+    my $self = shift;
+    $self->{mq}->disconnect()
+        if exists $self->{mq} && defined $self->{mq};
+}
+
 1;
 
 __END__
@@ -177,7 +184,7 @@ Log::Log4perl::Appender::RabbitMQ - Log to RabbitMQ
 
 =head1 VERSION
 
-version 0.141470
+version 0.200001
 
 =head1 SYNOPSIS
 
@@ -238,6 +245,14 @@ Defaults to localhost.
 
 =item heartbeat
 
+=item ssl
+
+=item ssl_verify_host
+
+=item ssl_cacert
+
+=item ssl_init
+
 =back
 
 =head3 Exchange Options
@@ -315,13 +330,23 @@ methods:
 
 =back
 
-=head1 AUTHOR
+=head1 AUTHORS
+
+=over 4
+
+=item *
 
 Trevor Little <bundacia@tjlittle.com>
 
+=item *
+
+Alexander Hartmaier <abraxxa@cpan.org>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Trevor Little.
+This software is copyright (c) 2017 by Alexander Hartmaier.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
